@@ -2,13 +2,12 @@
 # Copyright (c) 2001-2004 Divmod Inc.
 # See LICENSE for details.
 
-"""POP3 client protocol implementation
+"""
+POP3 client protocol implementation
 
 Don't use this module directly.  Use twisted.mail.pop3 instead.
 
-@author: U{Jp Calderone<mailto:exarkun@twistedmatrix.com>}
-
-API Stability: Unstable
+@author: Jp Calderone
 """
 
 import re, md5
@@ -179,12 +178,18 @@ class POP3Client(basic.LineOnlyReceiver, policies.TimeoutMixin):
         if self._blockedQueue == []:
             self._blockedQueue = None
         elif self._blockedQueue is not None:
-            d, f, a = self._blockedQueue.pop(0)
-            if not self._blockedQueue:
-                self._blockedQueue = None
+            _blockedQueue = self._blockedQueue
+            self._blockedQueue = None
 
+            d, f, a = _blockedQueue.pop(0)
             d2 = f(*a)
             d2.chainDeferred(d)
+            # f is a function which uses _blocked (otherwise it wouldn't
+            # have gotten into the blocked queue), which means it will have
+            # re-set _blockedQueue to an empty list, so we can put the rest
+            # of the blocked queue back into it now.
+            self._blockedQueue.extend(_blockedQueue)
+
 
     def sendShort(self, cmd, args):
         # Internal helper.  Send a command to which a short response
@@ -462,7 +467,7 @@ class POP3Client(basic.LineOnlyReceiver, policies.TimeoutMixin):
             d.addCallback(self._loginTLS, username, password)
             return d
 
-        elif self.startedTLS or self.allowInsecureLogin:
+        elif self.startedTLS or not nontlsTransport or self.allowInsecureLogin:
             return self._plaintext(username, password)
         else:
             return defer.fail(InsecureAuthenticationDisallowed())
