@@ -1,13 +1,11 @@
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
 """
-Test cases for twisted.mail.pop3 module.
+Test cases for Ltwisted.mail.pop3} module.
 """
 
 import StringIO
-import string
 import hmac
 import base64
 import itertools
@@ -256,9 +254,12 @@ Someone set up us the bomb!\015
         protocol =  MyVirtualPOP3()
         protocol.service = self.factory
         clientProtocol = MyPOP3Downloader()
-        loopback.loopback(protocol, clientProtocol)
-        self.failUnlessEqual(clientProtocol.message, self.message)
-        protocol.connectionLost(failure.Failure(Exception("Test harness disconnect")))
+        def check(ignored):
+            self.failUnlessEqual(clientProtocol.message, self.message)
+            protocol.connectionLost(
+                failure.Failure(Exception("Test harness disconnect")))
+        d = loopback.loopbackAsync(protocol, clientProtocol)
+        return d.addCallback(check)
     testLoopback.suppress = [util.suppress(message="twisted.mail.pop3.POP3Client is deprecated")]
 
 
@@ -268,24 +269,23 @@ class DummyPOP3(pop3.POP3):
     magic = '<moshez>'
 
     def authenticateUserAPOP(self, user, password):
-        return pop3.IMailbox, DummyMailbox(), lambda: None
+        return pop3.IMailbox, DummyMailbox(ValueError), lambda: None
+
+
 
 class DummyMailbox(pop3.Mailbox):
 
-    messages = [
-'''\
-From: moshe
-To: moshe
+    messages = ['From: moshe\nTo: moshe\n\nHow are you, friend?\n']
 
-How are you, friend?
-''']
-
-    def __init__(self):
+    def __init__(self, exceptionType):
         self.messages = DummyMailbox.messages[:]
+        self.exceptionType = exceptionType
 
     def listMessages(self, i=None):
         if i is None:
             return map(len, self.messages)
+        if i >= len(self.messages):
+            raise self.exceptionType()
         return len(self.messages[i])
 
     def getMessage(self, i):
@@ -293,7 +293,7 @@ How are you, friend?
 
     def getUidl(self, i):
         if i >= len(self.messages):
-            raise ValueError()
+            raise self.exceptionType()
         return str(i)
 
     def deleteMessage(self, i):
@@ -302,41 +302,71 @@ How are you, friend?
 
 class AnotherPOP3TestCase(unittest.TestCase):
 
-    def runTest(self, lines, expected):
+    def runTest(self, lines, expectedOutput):
         dummy = DummyPOP3()
-        client = LineSendingProtocol([
-            "APOP moshez dummy",
-            "LIST",
-            "UIDL",
-            "RETR 1",
-            "RETR 2",
-            "DELE 1",
-            "RETR 1",
-            "QUIT",
-        ])
-        expected_output = '+OK <moshez>\r\n+OK Authentication succeeded\r\n+OK 1\r\n1 44\r\n.\r\n+OK \r\n1 0\r\n.\r\n+OK 44\r\nFrom: moshe\r\nTo: moshe\r\n\r\nHow are you, friend?\r\n.\r\n-ERR Bad message number argument\r\n+OK \r\n-ERR message deleted\r\n+OK \r\n'
-        loopback.loopback(dummy, client)
-        self.failUnlessEqual(expected_output, '\r\n'.join(client.response) + '\r\n')
+        client = LineSendingProtocol(lines)
+        d = loopback.loopbackAsync(dummy, client)
+        return d.addCallback(self._cbRunTest, client, dummy, expectedOutput)
+
+
+    def _cbRunTest(self, ignored, client, dummy, expectedOutput):
+        self.failUnlessEqual('\r\n'.join(expectedOutput),
+                             '\r\n'.join(client.response))
         dummy.connectionLost(failure.Failure(Exception("Test harness disconnect")))
+        return ignored
 
 
-    def testBuffer(self):
-        lines = string.split('''\
-APOP moshez dummy
-LIST
-UIDL
-RETR 1
-RETR 2
-DELE 1
-RETR 1
-QUIT''', '\n')
-        expected_output = '+OK <moshez>\r\n+OK Authentication succeeded\r\n+OK 1\r\n1 44\r\n.\r\n+OK \r\n1 0\r\n.\r\n+OK 44\r\nFrom: moshe\r\nTo: moshe\r\n\r\nHow are you, friend?\r\n.\r\n-ERR Bad message number argument\r\n+OK \r\n-ERR message deleted\r\n+OK \r\n'
-        self.runTest(lines, expected_output)
+    def test_buffer(self):
+        """
+        Test a lot of different POP3 commands in an extremely pipelined
+        scenario.
+
+        This test may cover legitimate behavior, but the intent and
+        granularity are not very good.  It would likely be an improvement to
+        split it into a number of smaller, more focused tests.
+        """
+        return self.runTest(
+            ["APOP moshez dummy",
+             "LIST",
+             "UIDL",
+             "RETR 1",
+             "RETR 2",
+             "DELE 1",
+             "RETR 1",
+             "QUIT"],
+            ['+OK <moshez>',
+             '+OK Authentication succeeded',
+             '+OK 1',
+             '1 44',
+             '.',
+             '+OK ',
+             '1 0',
+             '.',
+             '+OK 44',
+             'From: moshe',
+             'To: moshe',
+             '',
+             'How are you, friend?',
+             '.',
+             '-ERR Bad message number argument',
+             '+OK ',
+             '-ERR message deleted',
+             '+OK '])
+
+
+    def test_noop(self):
+        """
+        Test the no-op command.
+        """
+        return self.runTest(
+            ['APOP spiv dummy',
+             'NOOP',
+             'QUIT'],
+            ['+OK <moshez>',
+             '+OK Authentication succeeded',
+             '+OK ',
+             '+OK '])
 
-    def testNoop(self):
-        lines = ['APOP spiv dummy', 'NOOP', 'QUIT']
-        expected_output = '+OK <moshez>\r\n+OK Authentication succeeded\r\n+OK \r\n+OK \r\n'
-        self.runTest(lines, expected_output)
 
     def testAuthListing(self):
         p = DummyPOP3()
@@ -347,9 +377,13 @@ QUIT''', '\n')
             "QUIT",
         ])
 
-        loopback.loopback(p, client)
+        d = loopback.loopbackAsync(p, client)
+        return d.addCallback(self._cbTestAuthListing, client)
+
+    def _cbTestAuthListing(self, ignored, client):
         self.failUnless(client.response[1].startswith('+OK'))
-        self.assertEquals(client.response[2:6], ["AUTH1", "SECONDAUTH", "AUTHLAST", "."])
+        self.assertEquals(client.response[2:6],
+                          ["AUTH1", "SECONDAUTH", "AUTHLAST", "."])
 
     def testIllegalPASS(self):
         dummy = DummyPOP3()
@@ -357,8 +391,11 @@ QUIT''', '\n')
             "PASS fooz",
             "QUIT"
         ])
+        d = loopback.loopbackAsync(dummy, client)
+        return d.addCallback(self._cbTestIllegalPASS, client, dummy)
+
+    def _cbTestIllegalPASS(self, ignored, client, dummy):
         expected_output = '+OK <moshez>\r\n-ERR USER required before PASS\r\n+OK \r\n'
-        loopback.loopback(dummy, client)
         self.failUnlessEqual(expected_output, '\r\n'.join(client.response) + '\r\n')
         dummy.connectionLost(failure.Failure(Exception("Test harness disconnect")))
 
@@ -368,8 +405,11 @@ QUIT''', '\n')
             "PASS ",
             "QUIT"
         ])
+        d = loopback.loopbackAsync(dummy, client)
+        return d.addCallback(self._cbTestEmptyPASS, client, dummy)
+
+    def _cbTestEmptyPASS(self, ignored, client, dummy):
         expected_output = '+OK <moshez>\r\n-ERR USER required before PASS\r\n+OK \r\n'
-        loopback.loopback(dummy, client)
         self.failUnlessEqual(expected_output, '\r\n'.join(client.response) + '\r\n')
         dummy.connectionLost(failure.Failure(Exception("Test harness disconnect")))
 
@@ -493,7 +533,7 @@ class GlobalCapabilitiesTestCase(unittest.TestCase):
 class TestRealm:
     def requestAvatar(self, avatarId, mind, *interfaces):
         if avatarId == 'testuser':
-            return pop3.IMailbox, DummyMailbox(), lambda: None
+            return pop3.IMailbox, DummyMailbox(ValueError), lambda: None
         assert False
 
 
@@ -527,7 +567,7 @@ class SASLTestCase(unittest.TestCase):
 
 
 
-class CommandTestCase(unittest.TestCase):
+class CommandMixin:
     """
     Tests for all the commands a POP3 server is allowed to receive.
     """
@@ -546,7 +586,7 @@ More message text for you.
         a transport that buffers output to a StringIO.
         """
         p = pop3.POP3()
-        p.mbox = DummyMailbox()
+        p.mbox = self.mailboxType(self.exceptionType)
         p.schedule = list
         self.pop3Server = p
 
@@ -876,6 +916,63 @@ More message text for you.
 
 
 
+_listMessageDeprecation = (
+    "twisted.mail.pop3.IMailbox.listMessages may not "
+    "raise IndexError for out-of-bounds message numbers: "
+    "raise ValueError instead.")
+_listMessageSuppression = util.suppress(
+    message=_listMessageDeprecation,
+    category=PendingDeprecationWarning)
+
+_getUidlDeprecation = (
+    "twisted.mail.pop3.IMailbox.getUidl may not "
+    "raise IndexError for out-of-bounds message numbers: "
+    "raise ValueError instead.")
+_getUidlSuppression = util.suppress(
+    message=_getUidlDeprecation,
+    category=PendingDeprecationWarning)
+
+class IndexErrorCommandTestCase(CommandMixin, unittest.TestCase):
+    """
+    Run all of the command tests against a mailbox which raises IndexError
+    when an out of bounds request is made.  This behavior will be deprecated
+    shortly and then removed.
+    """
+    exceptionType = IndexError
+    mailboxType = DummyMailbox
+
+    def testLISTWithBadArgument(self):
+        return CommandMixin.testLISTWithBadArgument(self)
+    testLISTWithBadArgument.suppress = [_listMessageSuppression]
+
+
+    def testUIDLWithBadArgument(self):
+        return CommandMixin.testUIDLWithBadArgument(self)
+    testUIDLWithBadArgument.suppress = [_getUidlSuppression]
+
+
+    def testTOPWithBadArgument(self):
+        return CommandMixin.testTOPWithBadArgument(self)
+    testTOPWithBadArgument.suppress = [_listMessageSuppression]
+
+
+    def testRETRWithBadArgument(self):
+        return CommandMixin.testRETRWithBadArgument(self)
+    testRETRWithBadArgument.suppress = [_listMessageSuppression]
+
+
+
+class ValueErrorCommandTestCase(CommandMixin, unittest.TestCase):
+    """
+    Run all of the command tests against a mailbox which raises ValueError
+    when an out of bounds request is made.  This is the correct behavior and
+    after support for mailboxes which raise IndexError is removed, this will
+    become just C{CommandTestCase}.
+    """
+    exceptionType = ValueError
+    mailboxType = DummyMailbox
+
+
 
 class SyncDeferredMailbox(DummyMailbox):
     """
@@ -887,14 +984,21 @@ class SyncDeferredMailbox(DummyMailbox):
 
 
 
-class SyncDeferredCommandTestCase(CommandTestCase):
+class IndexErrorSyncDeferredCommandTestCase(IndexErrorCommandTestCase):
     """
-    Run all of the L{CommandTestCase} tests with a synchronous-Deferred
-    returning IMailbox implementation.
+    Run all of the L{IndexErrorCommandTestCase} tests with a
+    synchronous-Deferred returning IMailbox implementation.
     """
-    def setUp(self):
-        CommandTestCase.setUp(self)
-        self.pop3Server.mbox = SyncDeferredMailbox()
+    mailboxType = SyncDeferredMailbox
+
+
+
+class ValueErrorSyncDeferredCommandTestCase(ValueErrorCommandTestCase):
+    """
+    Run all of the L{ValueErrorCommandTestCase} tests with a
+    synchronous-Deferred returning IMailbox implementation.
+    """
+    mailboxType = SyncDeferredMailbox
 
 
 
@@ -903,9 +1007,9 @@ class AsyncDeferredMailbox(DummyMailbox):
     Mailbox which has a listMessages implementation which returns a Deferred
     which has not yet fired.
     """
-    def __init__(self):
+    def __init__(self, *a, **kw):
         self.waiting = []
-        DummyMailbox.__init__(self)
+        DummyMailbox.__init__(self, *a, **kw)
 
 
     def listMessages(self, n=None):
@@ -916,15 +1020,12 @@ class AsyncDeferredMailbox(DummyMailbox):
 
 
 
-class AsyncDeferredCommandTestCase(CommandTestCase):
+class IndexErrorAsyncDeferredCommandTestCase(IndexErrorCommandTestCase):
     """
-    Run all of the L{CommandTestCase} tests with an asynchronous-Deferred
+    Run all of the L{IndexErrorCommandTestCase} tests with an asynchronous-Deferred
     returning IMailbox implementation.
     """
-    def setUp(self):
-        CommandTestCase.setUp(self)
-        self.pop3Server.mbox = AsyncDeferredMailbox()
-
+    mailboxType = AsyncDeferredMailbox
 
     def _flush(self):
         """
@@ -933,4 +1034,36 @@ class AsyncDeferredCommandTestCase(CommandTestCase):
         while self.pop3Server.mbox.waiting:
             d, a = self.pop3Server.mbox.waiting.pop()
             d.callback(a)
-        CommandTestCase._flush(self)
+        IndexErrorCommandTestCase._flush(self)
+
+
+
+class ValueErrorAsyncDeferredCommandTestCase(ValueErrorCommandTestCase):
+    """
+    Run all of the L{IndexErrorCommandTestCase} tests with an asynchronous-Deferred
+    returning IMailbox implementation.
+    """
+    mailboxType = AsyncDeferredMailbox
+
+    def _flush(self):
+        """
+        Fire whatever Deferreds we've built up in our mailbox.
+        """
+        while self.pop3Server.mbox.waiting:
+            d, a = self.pop3Server.mbox.waiting.pop()
+            d.callback(a)
+        ValueErrorCommandTestCase._flush(self)
+
+class POP3MiscTestCase(unittest.TestCase):
+    """
+    Miscellaneous tests more to do with module/package structure than
+    anything to do with the Post Office Protocol.
+    """
+    def test_all(self):
+        """
+        This test checks that all names listed in
+        twisted.mail.pop3.__all__ are actually present in the module.
+        """
+        mod = twisted.mail.pop3
+        for attr in mod.__all__:
+            self.failUnless(hasattr(mod, attr))
